<?php

/**
 * Batch update form.
 * @param $form
 * @param $form_state
 * @return mixed
 */
function brewery_db_beer_batch_import_beer_form($form, &$form_state) {
  return confirm_form($form, 'Import Beer', 'admin/config/services/brewery-db/beer', t('<p>This will import beer from BreweryDB using the !getBeers method into your system and create/update Beer entities.</p>', array('!getBeers' => l('getBeers()', 'http://www.brewerydb.com/developers/docs-endpoint/beer_index#1', array('external' => TRUE, 'attributes' => array('target' => '_blank'))))), 'Import Beer', 'Cancel');
}

/**
 * Form submit handler. Makes call to set up the batch.
 * @param $form
 * @param $form_state
 */
function brewery_db_beer_batch_import_beer_form_submit($form, &$form_state) {
  $batch = brewery_db_beer_batch_setup();
  batch_set($batch);
  $form_state['redirect'] = 'admin/config/services/brewery-db/beer/import-beer';
}

/**
 * Sets up the batch.
 */
function brewery_db_beer_batch_setup($start = 1, $num = 0, $since = 0) {

  $retrieve_brewery_info = variable_get('brewery_db_api_retrieve_brewery_information', FALSE) ? 'Y' : 'N';

  $client = new BreweryDBAPIBeer();
  $response = $client->getBeers($start, $retrieve_brewery_info, $since);

  if (!$num) {
    $end = $response->numberOfPages;
  }
  else {
    $end = $start + ($num - 1);
    if ($end > $response->numberOfPages) {
      $end = $response->numberOfPages;
    }
  }

  $operations = array();

  while ($start <= $end) {
    if (!isset($response)) {
      $response = $client->getBeers($start, $retrieve_brewery_info, $since);
    }
    $operations[] = array('brewery_db_beer_batch_record_page_start', array($start));
    foreach ($response->data as $beer) {
      $operations[] = array('brewery_db_beer_batch_process_beer', array($beer, $start));
    } // Loop thru beers.
    $operations[] = array('brewery_db_beer_batch_record_page_end', array($start));
    unset($response);
    $start++;
  } // Loop thru pages of results.

  if (!$num) {
    // Only update timestamp when doing full import (to end of results).
    $operations[] = array('brewery_db_beer_batch_record_last_sync_timestamp', array(time()));
  }

  $batch = array(
    'title' => t('Bartender!! Please pass down some beers from BreweryDB!'),
    'init_message' => t('Pouring some pints...'),
    'operations' => $operations,
    'finished' => 'brewery_db_beer_import_beer_batch_finished',
    'file' => drupal_get_path('module', 'brewery_db_beer') . '/includes/brewery_db_beer.batch.beer.inc',
  );

  return $batch;

}

/**
 * Update page number of last API call so we can know where to restart on fail.
 * @param $page_num
 * @param $context
 */
function brewery_db_beer_batch_record_page_start($page_num, &$context) {
  variable_set('brewery_db_beer_last_page_started', $page_num);
  $context['message'] = t('*** Starting page !page.', array('!page' => $page_num));
}

/**
 * Update page number of last API call so we can know where to restart on fail.
 * @param $page_num
 * @param $context
 */
function brewery_db_beer_batch_record_page_end($page_num, &$context) {
  variable_set('brewery_db_beer_last_page_completed', $page_num);
  $context['message'] = t('*** Completed page !page.', array('!page' => $page_num));
}

/**
 * This processes a beer item from BreweryDB.
 * @param $item
 * @param $context
 */
function brewery_db_beer_batch_process_beer($item, $page_num, &$context) {

  // @todo: abstract this out into a class method to reduce possible code duplication
  // @todo: a Feature will provide a fully populated/Fielded entity so we need mapper methods anyway.

  $query = new BreweryDBBeerQuery(array('entity_type' => 'brewery_db_beer'));
  $beer = $query->findByBeerId($item->id);

  if (!$beer) {
    $beer = entity_create('brewery_db_beer', array());
    $beer->is_new = TRUE;
  }

  $beer->beer_id = $item->id;
  $beer->name = $item->name;
  $beer->status = $item->status;
  $beer->status_display = $item->statusDisplay;
  $beer->data = serialize($item);

  entity_get_controller('brewery_db_beer')->save($beer);

  $context['message'] = t('Imbibing some @name', array('@name' => $item->name));
  $context['results'][] = $item->id;

}

/**
 * Update timestamp of last API call.
 * @param $timestamp
 * @param $context
 */
function brewery_db_beer_batch_record_last_sync_timestamp($timestamp, &$context) {
  variable_set('brewery_db_beer_last_sync', $timestamp);
  $context['message'] = t('Updated last API call timestamp to !timestamp.', array('!timestamp' => date('Y-m-d H:i:s', $timestamp)));
}

/**
 * Batch finished callback.
 * @param $success
 * @param $results
 * @param $operations
 */
function brewery_db_beer_import_beer_batch_finished($success, $results, $operations) {
  if (!empty($results['current_page'])) {
    // Remove this from the results (not needed for reporting).
    unset($results['current_page']);
  }
  if ($success) {
    $message = format_plural(count($results),
      '1 bottle of beer on the wall, 1 bottle of beer.',
      '@count bottles of beer on the wall, @count bottles of beer.');
    $message .= theme('item_list', $results);
    drupal_set_message($message);
  }
  else {
    $error_operation = reset($operations);
    $message = t('An error occurred while processing %error_operation with arguments: @arguments', array('%error_operation' => $error_operation[0], '@arguments' => print_r($error_operation[1], TRUE)));
    drupal_set_message($message, 'error');
  }
}
